package com.aliyun.iot.aep.demo;

import android.app.Activity;
import android.app.Application;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.res.AssetManager;
import android.net.Uri;
import android.os.Bundle;

import com.aliyun.iot.aep.component.router.IUrlHandler;
import com.aliyun.iot.aep.routerexternal.RouterExternal;
import com.aliyun.iot.aep.utils.ALog;

import org.json.JSONArray;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;

/**
 * Created by wuwang on 2017/11/9.
 */

public class BundleManager {
    static final private String TAG = "BundleManager";

    static public int init(Application app) {
        int ret = 0;

        ArrayList<String> files = _findAllBundleConfigureFiles(app);
        if (null == files || files.isEmpty())
            return ret;

        BundleConfigure configure = null;
        for (String file : files) {
            configure = _prepareConfigure(app, file);
            if (null == configure || null == configure.pages || configure.pages.isEmpty())
                continue;

            for (PageConfigure pageConfigure : configure.pages)
                _registerPage(app, pageConfigure);
        }

        return ret;
    }

    static private ArrayList<String> _findAllBundleConfigureFiles(final Application app) {
        ArrayList<String> ret = new ArrayList<>();

        try {
            AssetManager assetManager = app.getAssets();
            if (null == assetManager)
                return ret;

            String[] files = assetManager.list("");
            if (null == files || 0 >= files.length)
                return ret;

            for (String file : files) {
                if (null == file || 0 >= file.length() || false == file.endsWith("_config.json") || false == file.startsWith("bundle_"))
                    continue;
                ALog.d(TAG, "page-file: " + file);
                ret.add(file);
            }
        } catch (Exception ex) {
            ALog.e(TAG, "find-bundle-configure failed", ex);
        }

        return ret;
    }

    static private BundleConfigure _prepareConfigure(final Application app, final String filePath) {
        BundleConfigure ret = null;

        if (null == app || null == filePath || filePath.isEmpty())
            return ret;

        try {
            InputStream configureInputStream = app.getAssets().open(filePath);

            String body = readFile(configureInputStream);
            if (null == body || body.isEmpty())
                return ret;

            JSONObject jsonObject = new JSONObject(body);
            JSONArray pageJsonArray = (null != jsonObject ? jsonObject.getJSONArray("pages") : null);
            if (null == pageJsonArray)
                return ret;

            ret = new BundleConfigure();
            ret.name = jsonObject.optString("name");
            ret.pages = new ArrayList<>();

            JSONArray navigationJSONArray = null;
            for (int i = pageJsonArray.length() - 1; i >= 0; --i) {
                jsonObject = pageJsonArray.getJSONObject(i);

                navigationJSONArray = (null != jsonObject ? jsonObject.getJSONArray("navigationConfigures") : null);
                if (null == navigationJSONArray || 0 >= navigationJSONArray.length())
                    continue;

                PageConfigure pageConfigure = new PageConfigure();
                pageConfigure.name = jsonObject.optString("name");
                pageConfigure.version = jsonObject.optString("version");
                pageConfigure.needLogin = jsonObject.optBoolean("needLogin");
                pageConfigure.navigationConfigures = new ArrayList<>();

                NavigationConfigure navigationConfigure = null;
                for (int j = navigationJSONArray.length() - 1; j >= 0; --j) {
                    JSONObject navigationJsonObject = navigationJSONArray.getJSONObject(j);
                    if (null == navigationJsonObject)
                        continue;

                    navigationConfigure = new NavigationConfigure();
                    navigationConfigure.navigationCode = navigationJsonObject.optString("navigationCode");
                    navigationConfigure.navigationIntentAction = navigationJsonObject.optString("navigationIntentAction");
                    navigationConfigure.navigationIntentCategory = navigationJsonObject.optString("navigationIntentCategory");
                    navigationConfigure.navigationIntentUrl = navigationJsonObject.optString("navigationIntentUrl");

                    ALog.d(TAG, "prepare navigation-configure: " + navigationConfigure.navigationCode + ", " + navigationConfigure.navigationIntentUrl);
                    pageConfigure.navigationConfigures.add(navigationConfigure);
                }

                ret.pages.add(pageConfigure);
            }

        } catch (Exception ex) {
            ALog.e(TAG, "prepare-bundle-configure", ex);
        }


        return ret;
    }

    static private void _registerPage(final Application app, final PageConfigure configure) {
        if (null == configure || null == configure.navigationConfigures)
            return;

        ArrayList<String> nativeUrls = new ArrayList<>();
        ArrayList<NavigationConfigure> configures = new ArrayList<>();

        NavigationConfigure deepCopyItem = null;
        for (NavigationConfigure item : configure.navigationConfigures) {
            if (null == item.navigationCode || item.navigationCode.isEmpty() || null == item.navigationIntentUrl || item.navigationIntentUrl.isEmpty())
                continue;

            deepCopyItem = new NavigationConfigure();
            deepCopyItem.navigationCode = item.navigationCode;
            deepCopyItem.navigationIntentUrl = item.navigationIntentUrl;
            deepCopyItem.navigationIntentAction = item.navigationIntentAction;
            deepCopyItem.navigationIntentCategory = item.navigationIntentCategory;

            configures.add(deepCopyItem);
            nativeUrls.add(deepCopyItem.navigationIntentUrl);

            ALog.d(TAG, "register-native-page: " + item.navigationCode + ", " + item.navigationIntentUrl);

//            RouterExternal.getInstance().init(app, "test");
            RouterExternal.getInstance().registerNativeCodeUrl(deepCopyItem.navigationCode, deepCopyItem.navigationIntentUrl);
            RouterExternal.getInstance().registerNativePages(nativeUrls, new NativeUrlHandler(deepCopyItem));
        }
    }

    /* methods: helper */

    static private String readFile(InputStream configureInputStream) {
        StringBuilder ret = null;

        BufferedReader reader = null;
        try {
            reader = new BufferedReader(new InputStreamReader(configureInputStream));

            StringBuilder stringBuilder = new StringBuilder();
            String line = null;
            while (null != (line = reader.readLine())) {
                stringBuilder.append(line).append("\n");
            }

            ret = stringBuilder;
        } catch (Exception ex) {
            ALog.e(TAG, "read-file", ex);
        }

        try {
            if (null != reader)
                reader.close();
        } catch (Exception ex) {
            ALog.e(TAG, "read-file", ex);
        }

        return null != ret ? ret.toString() : null;
    }

    /* inner type */

    static final private class BundleConfigure {
        public String name;
        public String description;
        public ArrayList<PageConfigure> pages;
    }

    static final private class PageConfigure {
        public String name;
        public String version;
        public boolean needLogin;
        public ArrayList<NavigationConfigure> navigationConfigures;
    }

    static final private class NavigationConfigure {
        public String navigationCode;
        public String navigationIntentAction;
        public String navigationIntentCategory;
        public String navigationIntentUrl;
    }

    static final private class NativeUrlHandler implements IUrlHandler {

        private final NavigationConfigure navigationConfigure;

        NativeUrlHandler(NavigationConfigure configures) {
            this.navigationConfigure = configures;
        }

        @Override
        public void onUrlHandle(Context context, String url, Bundle bundle, boolean startActForResult, int reqCode) {
            ALog.d(TAG, "onUrlHandle: url: " + url);
            if (null == context || null == url || url.isEmpty())
                return;

            /* prepare the intent */
            Intent intent = new Intent();
            intent.setData(Uri.parse(url));

            if (null != this.navigationConfigure.navigationIntentAction)
                intent.setAction(this.navigationConfigure.navigationIntentAction);
            if (null != this.navigationConfigure.navigationIntentCategory)
                intent.addCategory(this.navigationConfigure.navigationIntentCategory);


            /* start the navigated activity */
            ALog.d(TAG, "startActivity(): url: " + this.navigationConfigure.navigationIntentUrl + ", startActForResult: " + startActForResult + ", reqCode: " + reqCode);
            this.startActivity(context, intent, bundle, startActForResult, reqCode);
        }

        private void startActivity(Context context, Intent intent, Bundle bundle, boolean startActForResult, int reqCode) {
            if (null == context || null == intent)
                return;


            /* startActivityForResult() 场景，只能被 Activity 调用 */
            if (startActForResult) {
                if (false == (context instanceof Activity))
                    return;

                ((Activity) context).startActivityForResult(intent, reqCode, bundle);

                return;
            }

            /* startActivity 被 Application 调用时的处理 */
            if (context instanceof Application) {
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                context.startActivity(intent, bundle);
            }
            /* startActivity 被 Activity、Service 调用时的处理 */
            else if (context instanceof Activity || context instanceof Service) {
                context.startActivity(intent, bundle);
            }
            /* startActivity 被其他组件调用时的处理 */
            else {
                // 暂不支持
            }
        }

    }

}
