package com.aliyun.iot.aep.demo;

import android.app.Activity;
import android.app.Application;
import android.os.Bundle;
import android.util.Log;

import com.alibaba.sdk.android.push.CloudPushService;
import com.alibaba.sdk.android.push.CommonCallback;
import com.alibaba.sdk.android.push.noonesdk.PushServiceFactory;
import com.aliyun.alink.linksdk.tools.ThreadTools;
import com.aliyun.iot.aep.demo.sdk.base.delegate.APIGatewaySDKDelegate;
import com.aliyun.iot.aep.demo.sdk.base.delegate.OpenAccountSDKDelegate;
import com.aliyun.iot.aep.sdk.framework.AApplication;
import com.aliyun.iot.aep.sdk.framework.sdk.SDKConfigure;

/**
 * Created by wuwang on 2017/10/30.
 */

public class DemoApplication extends AApplication {

    @Override
    public void onCreate() {
        super.onCreate();

        initPush(this);
        String packageName = this.getPackageName();
        if (!packageName.equals(ThreadTools.getProcessName(this, android.os.Process.myPid()))) {
            return;
        }
        this.init();
    }

    /* methods: helper init */

    private void init() {

        /* 环境变量准备 */
        EnvConfigure.putEnvArg(EnvConfigure.KEY_IS_DEBUG, "true");
        EnvConfigure.putEnvArg(EnvConfigure.KEY_PRODUCT_ID, "3440950");

        /* 加载Native页面 */
        BundleManager.init(this);

        /* 初始化 SDK */
        this._initBaseSdk();

        this._initOtherSdk();

        this.registerActivityLifecycleCallbacks(new DemoAppActivityLifeCycle(this));
    }

    private void _initBaseSdk() {
        SDKManager.Result result = null;

        // API网关
        {
            SDKConfigure configure = new SDKConfigure("API-Client", "0.0.1", APIGatewaySDKDelegate.class.getName());

            result = new SDKManager.Result();
            result.sdkName = "APIGateway";
            result.sdkVer = "*";
            result.bInitialized = true;
            result.resultCode = new APIGatewaySDKDelegate().init(this, configure, EnvConfigure.envArgs);
            SDKManager.InitResultHolder.updateResult(APIGatewaySDKDelegate.class.getName(), result);
        }

        // 帐号
        {
            SDKConfigure configure = new SDKConfigure("OpenAccount", "0.0.1", APIGatewaySDKDelegate.class.getName());

            result = new SDKManager.Result();
            result.sdkName = "OpenAccount";
            result.sdkVer = "*";
            result.bInitialized = true;
            result.resultCode = new OpenAccountSDKDelegate().init(this, configure, EnvConfigure.envArgs);
            SDKManager.InitResultHolder.updateResult(OpenAccountSDKDelegate.class.getName(), result);
        }
    }

    private void _initOtherSdk() {
        SDKManager.prepareForInitSdk(this);

        SDKManager.init_outOfUiThread(this);
        SDKManager.init_underUiThread(this);
    }

    /* methods: AApplication */

    @Override
    protected void onFirstActivityOnCreated(Activity activity, Bundle savedInstanceState) {
        super.onFirstActivityOnCreated(activity, savedInstanceState);
    }

    @Override
    protected void onFirstActivityOnVisible(Activity activity) {
        super.onFirstActivityOnVisible(activity);
    }

    @Override
    protected void onLastActivityDestroyed(Activity activity) {
        super.onLastActivityDestroyed(activity);
    }

    @Override
    protected void onLastActivityInvisible(Activity activity) {
        super.onLastActivityInvisible(activity);
    }

    void initPush(Application app) {
        PushServiceFactory.init(app);
        CloudPushService pushService = PushServiceFactory.getCloudPushService();
        pushService.setSecurityGuardAuthCode("114d");
        pushService.register(app, new CommonCallback() {
            @Override
            public void onSuccess(String response) {
                Log.d("aep-demo", "init cloudchannel success");
            }

            @Override
            public void onFailed(String errorCode, String errorMessage) {
                Log.d("aep-demo", "init cloudchannel failed -- errorcode:" + errorCode + " -- errorMessage:" + errorMessage);
            }
        });
    }
}
