package com.aliyun.iot.aep.demo.page.scan;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import com.aliyun.iot.aep.component.scan.ScanManager;
import com.aliyun.iot.aep.component.scan.ScanPluginBean;
import com.aliyun.iot.aep.demo.utils.ScreenTools;
import com.aliyun.iot.demo.R;
import com.aliyun.iot.aep.sdk.scan.manager.OnDecodeOverListener;
import com.aliyun.iot.aep.utils.ALog;

import java.util.ArrayList;

/**
 * Created by wb-zyl208210 on 2017/11/1.
 * demo Activity
 */
public class ScanActivity extends Activity implements View.OnClickListener {
    private static final String TAG = "ScanActivity";
    private static final String KEY_ISNEED_RESULT = "KEY_ISNEED_RESULT";
    private static final int RESULT_CODE_SUCCESS = 100;
    private static final int RESULT_CODE_FAILED = 200;

    private View mBack;
    private TextView mAlbum;
    private ScanFragment mScanFrament;
    private ViewScanView mViewScanView;
    private boolean mIsNeedResult;
    private ArrayList<String> mPluginNameList;
    private ArrayList<ScanPluginBean> mScanPluginBeans;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.scan_activity);
        initArgument();
        initView();
        initEvent();
    }

    //初始化参数
    private void initArgument() {
        Intent intent = getIntent();
        mIsNeedResult = intent.getBooleanExtra(KEY_ISNEED_RESULT, true);
        try {
            mScanPluginBeans = (ArrayList) intent.getParcelableArrayListExtra(ScanManager.PLUGIN_NAME_LIST);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void initView() {
        String scanTipStr = "将二维码放入框内，即可自动扫描";
        mScanFrament = (ScanFragment) getFragmentManager().findFragmentById(R.id.scan_fragment);

        mViewScanView = new ViewScanView(this);
        mViewScanView.setCornerBoundaryStyle(Color.parseColor("#00C1DE"), 8f, 0.05f);
        mViewScanView.setBoundaryStyle(Color.parseColor("#CCCCCC"), 2f);
        mViewScanView.setScanStyle(ViewScanView.style_radar);
        //mViewScanView.setScanStyleDrawableLine(R.mipmap.scanline,15);
        mViewScanView.setScancolor(Color.parseColor("#3f00C1DE"));
        mViewScanView.setScanTip(scanTipStr, (float) ScreenTools.convertDp2Px(this, 40f), Color.WHITE, (int) ScreenTools.convertDp2Px(this, 14f));

        mScanFrament.scanAreaVerticalPosition(0.25f);
        mScanFrament.setViewScanView(mViewScanView);
        mBack = findViewById(R.id.scanactivity_back_linearlayout);
        mAlbum = (TextView) findViewById(R.id.scanactivity_album_textview);
    }

    private void initEvent() {
        mAlbum.setOnClickListener(this);
        mBack.setOnClickListener(this);
        mScanFrament.setOnDecodeOverListener(new MyOnDecodeOverListener());
    }

    private void finishWithRequest(int resultCode, String barCode) {
        //如果有设置使用plugin处理使用调用plugin
        //        if (mScanPluginBeans != null && barCode != null) {
        if (!TextUtils.isEmpty(barCode)) {
            ScanManager.getInstance().executePlugin(this, barCode, mScanPluginBeans);
        }
        //如果需要返回结果则返回结果，此时同时finish掉当前页面
        if (mIsNeedResult) {
            Intent result = new Intent();
            result.putExtra("barCode", barCode);
            this.setResult(resultCode, result);
            this.finish();
        }

    }


    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.scanactivity_album_textview:
                mScanFrament.startPickPicFromLocalAlbum();//open the album ,then decode the pic whitch  you pick up
                break;
            case R.id.scanactivity_back_linearlayout:
                this.finish();
                break;
            default:

                break;
        }
    }

    class MyOnDecodeOverListener implements OnDecodeOverListener {

        //解析结果处理
        @Override
        public void onDecodeSucessFormPic(String barcode) {
            ALog.d(TAG, barcode);
            Toast.makeText(getApplicationContext(), barcode, Toast.LENGTH_SHORT).show();
            //mScanFrament.restartPreviewDecode(0);

            finishWithRequest(RESULT_CODE_SUCCESS, barcode);
        }

        @Override
        public void onDecodeSucess(String barcode) {
            ALog.d(TAG, barcode);
            Toast.makeText(getApplicationContext(), barcode, Toast.LENGTH_SHORT).show();
            //ScanManager.getInstance().executePlugin()
            //mScanFrament.restartPreviewDecode(0);

            finishWithRequest(RESULT_CODE_SUCCESS, barcode);
        }

        @Override
        public void onDecodeFailFromPic() {
            ALog.d(TAG, "pic decode fail!");
            String des = "该图片未发现二维码信息!";
            Toast.makeText(getApplicationContext(), des, Toast.LENGTH_SHORT).show();
            finishWithRequest(RESULT_CODE_FAILED, null);
        }

        @Override
        public void onDecodeFail() {

        }
    }
}
