package com.aliyun.iot.aep.demo.page.scan;

import android.animation.ValueAnimator;
import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.LinearGradient;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.Rect;
import android.graphics.drawable.Drawable;
import android.text.TextPaint;
import android.text.TextUtils;
import android.util.AttributeSet;
import android.util.Log;
import android.view.animation.DecelerateInterpolator;

import com.aliyun.iot.aep.sdk.scan.manager.camera.CameraManager;
import com.aliyun.iot.aep.sdk.scan.scansurface.AScanView;


/**
 * Created by wb-zyl208210 on 2017/11/1.
 * this is the view of scan area,and this view need get the Area(Rect) from the CameraManager,this is why it need extend from AScanView
 */
public final class ViewScanView extends AScanView {

    public static final int style_gridding = 0;//扫描区域的样式
    public static final int style_radar = 1;
    public static final int style_hybrid = 2;
    public static final int style_selfscanline = 3;

    private CameraManager cameraManager;//相机管理器，拿取最佳的扫描区域
    private Rect mFrame;//最佳扫描区域的Rect
    private Paint mLinePaint;//边框画笔
    private Paint mCornerLinePaint;//脚线画笔
    private Paint mScanPaint_Gridding;//网格样式画笔
    private Paint mScanPaint_Radio;//雷达样式画笔

    private int mScancolor;//扫描颜色

    private int mBoundaryColor;
    private float mBoundaryStrokeWidth;//扫描区域边线样式-线宽

    private LinearGradient mLinearGradient_Radar;//雷达样式的画笔shader

    private Path mGriddingPath;//网格样式的path
    private LinearGradient mLinearGradient_Gridding;//网格画笔的shader
    private float mGriddingLineWidth;//网格线的线宽，单位pix
    private int mGriddingDensity;//网格样式的，网格密度，值越大越密集

    private Matrix mScanMatrix;//变换矩阵，用来实现动画效果
    private ValueAnimator mValueAnimator;//值动画，用来变换矩阵操作
    private int mScanAnimatorDuration;//值动画的时长

    private int mScanStyle = 2;//网格 0：网格，1：纵向雷达 2:综合

    private float mCornerLineLenRatio;//扫描边框角线占边总长的比例
    private float mCornerLineLen;//根据比例计算的边框长度，从四角定点向临近的定点画出的长度
    private Path mCornerBoundaryLinePath;//边框path
    private int mCornerBoundaryColor;//角边框颜色
    private float mCornerBoundaryStrokeWidth;//角边框宽度


    private Drawable mSelfScanLineDrawable;//扫描线的图片--设置该图片则默认为-style_selfscanline
    private int mSelfScanDraLineHeight;//扫描线图高度
    private Rect mSelfScanDraLineRect;//自定义扫描线的bounds，即将绘制的区域

    private String mScanTip;
    private float tipMargin2ScanBottom;
    private TextPaint mTipTextPaint;
    private float mTipLength;

    public ViewScanView(Context context) {
        this(context, null);
    }

    // This constructor is used when the class is built from an XML resource.
    public ViewScanView(Context context, AttributeSet attrs) {
        this(context, attrs, 0);

    }

    public ViewScanView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init();
    }

    private void init() {

        mBoundaryColor = Color.TRANSPARENT;
        mBoundaryStrokeWidth = 2f;//px

        mCornerBoundaryColor = Color.WHITE;
        mCornerBoundaryStrokeWidth = 8f;//扫描区域边线样式-线宽nce rather than calling them every time in onDraw().

        mScanAnimatorDuration = 1800;

        mCornerLineLen = 50f;
        mCornerLineLenRatio = 0.06f;

        mGriddingDensity = 40;
        mGriddingLineWidth = 2;

        if (mTipTextPaint == null) {
            mTipTextPaint = new TextPaint();
            mTipTextPaint.setColor(Color.WHITE);
            mTipTextPaint.setTextSize(20);
        }

        mScanPaint_Gridding = new Paint(Paint.ANTI_ALIAS_FLAG);
        mScanPaint_Gridding.setStyle(Paint.Style.STROKE);
        mScanPaint_Gridding.setStrokeWidth(mGriddingLineWidth);

        mScanPaint_Radio = new Paint(Paint.ANTI_ALIAS_FLAG);
        mScanPaint_Radio.setStyle(Paint.Style.FILL);
        mScancolor = Color.parseColor("#3f99ee00");


        //变换矩阵，用来处理扫描的上下扫描效果
        mScanMatrix = new Matrix();
        mScanMatrix.setTranslate(0, 30);
    }

    public void setCameraManager(CameraManager cameraManager) {
        this.cameraManager = cameraManager;
    }

    @SuppressLint("DrawAllocation")
    @Override
    public void onDraw(Canvas canvas) {
        if (cameraManager == null || cameraManager.getFramingRect() == null) {
            return; // not ready yet, early draw before done configuring
        }
        if (mFrame == null) {
            mFrame = cameraManager.getFramingRect();
        }

        initCornerBoundaryStyle();
        initBoundaryStyle();
        canvas.drawRect(mFrame, mLinePaint);//先绘制 边框线
        canvas.drawPath(mCornerBoundaryLinePath, mCornerLinePaint);//再绘制，脚线
        switch (mScanStyle) {
            case style_gridding:
            default:
                initGriddingPathAndStyle();
                initScanValueAnim(-mFrame.height(), 0);
                canvas.drawPath(mGriddingPath, mScanPaint_Gridding);
                break;
            case style_radar:
                initRadarStyle();
                initScanValueAnim(-mFrame.height(), 0);
                canvas.drawRect(mFrame, mScanPaint_Radio);
                break;
            case style_hybrid:
                initGriddingPathAndStyle();
                initRadarStyle();
                initScanValueAnim(-mFrame.height(), 0);
                canvas.drawPath(mGriddingPath, mScanPaint_Gridding);
                canvas.drawRect(mFrame, mScanPaint_Radio);
                break;
            case style_selfscanline:
                initSelfScanLineStyle();
                initScanValueAnim(mFrame.top, mFrame.bottom - mSelfScanDraLineHeight);
                if (mSelfScanLineDrawable != null) {
                    mSelfScanLineDrawable.draw(canvas);
                }
                Log.d("draw", "onDraw: ");
                break;

        }
        if (!TextUtils.isEmpty(mScanTip) && mTipTextPaint != null) {
            canvas.drawText(mScanTip, (getMeasuredWidth() - mTipLength) / 2, mFrame.bottom + tipMargin2ScanBottom, mTipTextPaint);
        }

    }

    private void initBoundaryStyle() {
        if (mLinePaint == null) {
            //扫描区域的四角线框的样式
            mLinePaint = new Paint(Paint.ANTI_ALIAS_FLAG);
            mLinePaint.setColor(mBoundaryColor);
            mLinePaint.setStrokeWidth(mBoundaryStrokeWidth);
            mLinePaint.setStyle(Paint.Style.STROKE);
            mLinePaint.setColor(mBoundaryColor);
            mLinePaint.setStrokeWidth(mBoundaryStrokeWidth);
        }
    }

    //初始化定点角的样式
    private void initCornerBoundaryStyle() {
        if (mCornerBoundaryLinePath == null) {
            mCornerLineLen = mFrame.width() * mCornerLineLenRatio;
            int offset = (int) (mCornerBoundaryStrokeWidth / 2);
            Rect boundaryRect = new Rect(mFrame.left - offset, mFrame.top - offset, mFrame.right + offset, mFrame.bottom + offset);
            mCornerBoundaryLinePath = new Path();
            mCornerBoundaryLinePath.moveTo(boundaryRect.left, boundaryRect.top + mCornerLineLen);
            mCornerBoundaryLinePath.lineTo(boundaryRect.left, boundaryRect.top);
            mCornerBoundaryLinePath.lineTo(boundaryRect.left + mCornerLineLen, boundaryRect.top);
            mCornerBoundaryLinePath.moveTo(boundaryRect.right - mCornerLineLen, boundaryRect.top);
            mCornerBoundaryLinePath.lineTo(boundaryRect.right, boundaryRect.top);
            mCornerBoundaryLinePath.lineTo(boundaryRect.right, boundaryRect.top + mCornerLineLen);
            mCornerBoundaryLinePath.moveTo(boundaryRect.right, boundaryRect.bottom - mCornerLineLen);
            mCornerBoundaryLinePath.lineTo(boundaryRect.right, boundaryRect.bottom);
            mCornerBoundaryLinePath.lineTo(boundaryRect.right - mCornerLineLen, boundaryRect.bottom);
            mCornerBoundaryLinePath.moveTo(boundaryRect.left + mCornerLineLen, boundaryRect.bottom);
            mCornerBoundaryLinePath.lineTo(boundaryRect.left, boundaryRect.bottom);
            mCornerBoundaryLinePath.lineTo(boundaryRect.left, boundaryRect.bottom - mCornerLineLen);
        }
        if (mCornerLinePaint == null) {
            mCornerLinePaint = new Paint(Paint.ANTI_ALIAS_FLAG);
            mCornerLinePaint.setColor(mCornerBoundaryColor);
            mCornerLinePaint.setStrokeWidth(mCornerBoundaryStrokeWidth);
            mCornerLinePaint.setStyle(Paint.Style.STROKE);
            mCornerLinePaint.setColor(mCornerBoundaryColor);
            mCornerLinePaint.setStrokeWidth(mCornerBoundaryStrokeWidth);
        }
    }

    private void initRadarStyle() {
        if (mLinearGradient_Radar == null) {
            mLinearGradient_Radar = new LinearGradient(0, mFrame.top, 0, mFrame.bottom + 0.01f * mFrame.height(), new int[]{Color.TRANSPARENT, Color.TRANSPARENT, mScancolor, Color.TRANSPARENT}, new float[]{0, 0.85f, 0.99f, 1f}, LinearGradient.TileMode.CLAMP);
            mLinearGradient_Radar.setLocalMatrix(mScanMatrix);
            mScanPaint_Radio.setShader(mLinearGradient_Radar);
        }
    }

    public void initSelfScanLineStyle() {
        if (mSelfScanLineDrawable != null && mSelfScanDraLineRect == null) {
            mSelfScanDraLineHeight = mSelfScanDraLineHeight == 0 ? 50 : mSelfScanDraLineHeight;
            mSelfScanDraLineRect = new Rect(mFrame.left, mFrame.top, mFrame.right, mFrame.top + mSelfScanDraLineHeight);
            mSelfScanLineDrawable.setBounds(mSelfScanDraLineRect);
        }
    }

    private void initGriddingPathAndStyle() {
        if (mGriddingPath == null) {
            mGriddingPath = new Path();
            float wUnit = mFrame.width() / (mGriddingDensity + 0f);
            float hUnit = mFrame.height() / (mGriddingDensity + 0f);
            for (int i = 0; i <= mGriddingDensity; i++) {
                mGriddingPath.moveTo(mFrame.left + i * wUnit, mFrame.top);
                mGriddingPath.lineTo(mFrame.left + i * wUnit, mFrame.bottom);
            }
            for (int i = 0; i <= mGriddingDensity; i++) {
                mGriddingPath.moveTo(mFrame.left, mFrame.top + i * hUnit);
                mGriddingPath.lineTo(mFrame.right, mFrame.top + i * hUnit);
            }
        }
        if (mLinearGradient_Gridding == null) {
            mLinearGradient_Gridding = new LinearGradient(0, mFrame.top, 0, mFrame.bottom + 0.01f * mFrame.height(), new int[]{Color.TRANSPARENT, Color.TRANSPARENT, mScancolor, Color.TRANSPARENT}, new float[]{0, 0.5f, 0.99f, 1f}, LinearGradient.TileMode.CLAMP);
            mLinearGradient_Gridding.setLocalMatrix(mScanMatrix);
            mScanPaint_Gridding.setShader(mLinearGradient_Gridding);

        }
    }

    public void initScanValueAnim(int startVal, int endVal) {
        if (mValueAnimator == null) {
            mValueAnimator = new ValueAnimator();
            mValueAnimator.setDuration(mScanAnimatorDuration);
            mValueAnimator.setFloatValues(startVal, endVal);
            mValueAnimator.setRepeatMode(ValueAnimator.RESTART);
            mValueAnimator.setInterpolator(new DecelerateInterpolator());
            mValueAnimator.setRepeatCount(Integer.MAX_VALUE);
            mValueAnimator.addUpdateListener(new ValueAnimator.AnimatorUpdateListener() {
                @Override
                public void onAnimationUpdate(ValueAnimator animation) {
                    if (mScanMatrix != null) {
                        float animatedValue = (float) animation.getAnimatedValue();
                        mScanMatrix.setTranslate(0, animatedValue);
                        if (mLinearGradient_Gridding != null) {
                            mLinearGradient_Gridding.setLocalMatrix(mScanMatrix);
                        }
                        if (mLinearGradient_Radar != null) {
                            mLinearGradient_Radar.setLocalMatrix(mScanMatrix);
                        }
                        if (mSelfScanDraLineRect != null) {
                            mSelfScanDraLineRect.set(mSelfScanDraLineRect.left, (int) animatedValue, mSelfScanDraLineRect.right, (int) (animatedValue + mSelfScanDraLineHeight));
                            mSelfScanLineDrawable.setBounds(mSelfScanDraLineRect);
                        }
                        //mScanPaint.setShader(mLinearGradient); 不是必须的设置到shader即可
                        //Log.d("anim", "onAnimationUpdate: " + animatedValue);
                        invalidate();
                    }
                }
            });
            mValueAnimator.start();
        }
    }

    @Override
    protected void onDetachedFromWindow() {
        if (mValueAnimator != null && mValueAnimator.isRunning()) {
            mValueAnimator.cancel();
        }
        super.onDetachedFromWindow();
    }

    //设定扫描的颜色
    public void setScancolor(int colorValue) {
        this.mScancolor = colorValue;
    }

    public void setScanAnimatorDuration(int duration) {
        this.mScanAnimatorDuration = duration;
    }

    /*
    * @params colorValue:扫描区域角边框的颜色，strokeWidth：扫描区域外边框的线宽，cornerLineLen：扫描区域外边框四角的线长比例
    * */
    public void setCornerBoundaryStyle(int colorValue, float strokeWidth, float cornerLineLenRatio) {
        this.mCornerBoundaryColor = colorValue;
        this.mCornerBoundaryStrokeWidth = strokeWidth;
        this.mCornerLineLenRatio = cornerLineLenRatio;
    }

    /*
    * 扫描区域的样式
    * */
    public void setScanStyle(int scanStyle) {
        this.mScanStyle = scanStyle;
    }


    /*
    * 设置drawable的扫描线，设置后则自动设定当前的扫描样式 ： 3
    * */
    public void setScanStyleDrawableLine(int scanLineDrawableRes, int lineHeight) {
        mSelfScanLineDrawable = getContext().getResources().getDrawable(scanLineDrawableRes);
        mSelfScanDraLineHeight = lineHeight;
        setScanStyle(style_selfscanline);
    }

    /*
    * 扫描区域网格的样式
    * */
    public void setScanGriddingStyle(float strokeWidh, int density) {
        this.mGriddingLineWidth = strokeWidh;
        this.mGriddingDensity = density;
    }

    /*
    * @params colorValue:扫描区域边框的颜色，strokeWidth：扫描区域外边框的线宽
    * */
    public void setBoundaryStyle(int colorValue, float strokeWidth) {
        this.mBoundaryColor = colorValue;
        this.mBoundaryStrokeWidth = strokeWidth;
    }

    //设置扫描框下方的文字提示，不设置则不显示
    public void setScanTip(String tip, float margin2ScanBottom, int textColor, int textSize) {
        this.mScanTip = tip;
        this.tipMargin2ScanBottom = margin2ScanBottom;
        if (mTipTextPaint == null) {
            mTipTextPaint = new TextPaint();
        }
        mTipTextPaint.setColor(textColor);
        mTipTextPaint.setTextSize(textSize);
        mTipLength = mTipTextPaint.measureText(tip);
    }
}
