package com.aliyun.iot.aep.demo.sdk.delegate;

import android.app.Application;
import android.text.TextUtils;

import com.alibaba.fastjson.JSON;
import com.aliyun.alink.alirn.dev.BoneDevHelper;
import com.aliyun.alink.page.rn.InitializationHelper;
import com.aliyun.alink.sdk.rn.external.viewmanagers.iconfont.BoneReactTextShadowNode;
import com.aliyun.iot.aep.component.bundlemanager.ocache.BundleManager;
import com.aliyun.iot.aep.component.scan.ScanManager;
import com.aliyun.iot.aep.demo.EnvConfigure;
import com.aliyun.iot.aep.demo.sdk.BoneMobileScanPlugin;
import com.aliyun.iot.aep.demo.sdk.base.delegate.APIGatewaySDKDelegate;
import com.aliyun.iot.aep.routerexternal.PluginConfigManager;
import com.aliyun.iot.aep.routerexternal.RouterExternal;
import com.aliyun.iot.aep.sdk.framework.sdk.SDKConfigure;
import com.aliyun.iot.aep.sdk.framework.sdk.SimpleSDKDelegateImp;
import com.facebook.react.FrescoPackage;

import java.util.Map;

/**
 * RN容器
 */
public class RNContainerComponentDelegate extends SimpleSDKDelegateImp {

    private final static String TAG = "RNContainerComponentDelegate";

    @Override
    public int init(final Application application, SDKConfigure sdkConfigure, Map<String, String> map) {
        String pluginEnv = BoneDevHelper.readPluginEnv(application, "test");
        map.put(EnvConfigure.KEY_RN_CONTAINER_PLUGIN_ENV, pluginEnv);
        // 初始化路由
        RouterExternal.getInstance().init(application, pluginEnv);

        // 初始化包管理器
        {
            BundleManager.instance(application).setPluginEnv(pluginEnv);
            PluginConfigManager.getInstance().init(application);
            PluginConfigManager.getInstance().addPluginConfigChangeListener(new PluginConfigManager.OnPluginConfigChangedListener() {
                @Override
                public void onChange(PluginConfigManager.PluginConfigData pluginConfig) {
                    if (null != pluginConfig) {
                        BundleManager.instance(application).notifyUpdate(JSON.toJSONString(pluginConfig));
                    }
                }
            });
            PluginConfigManager.PluginConfigData data = PluginConfigManager.getInstance().getPluginConfigData();
            if (null != data) {
                BundleManager.instance(application).notifyUpdate(JSON.toJSONString(data));
            }
        }

        // 读取环境变量
        String serverEnv = map.get(APIGatewaySDKDelegate.ENV_KEY_API_CLIENT_API_ENV);
        if (TextUtils.isEmpty(serverEnv)) {
            serverEnv = "production"; // default is "production"
        } else if ("release".equalsIgnoreCase(serverEnv)) {
            serverEnv = "production";
        } else if ("pre".equalsIgnoreCase(serverEnv)) {
            serverEnv = "test";
        } else if ("test".equalsIgnoreCase(serverEnv)) {
            serverEnv = "development";
        }

        // 初始化 BoneMobile 容器
        InitializationHelper.initialize(application, pluginEnv, serverEnv);
        // 添加基于 Fresco 的图片组件支持
        InitializationHelper.addPackage(new FrescoPackage());

        // 配置 iconfont 字体
        BoneReactTextShadowNode.setFontFamilyForIconFont("demoapp_iconfont_20171130");

        // 支持扫码调试
        ScanManager.getInstance().registerPlugin("boneMobile", new BoneMobileScanPlugin());

        return 0;
    }

}
