package com.aliyun.iot.aep.utils;

import android.util.Log;

public class ALog {
    static public final byte LEVEL_DEBUG = 0X01;
    static public final byte LEVEL_INFO = 0X02;
    static public final byte LEVEL_WARNING = 0X03;
    static public final byte LEVEL_ERROR = 0X04;

    static public byte level = LEVEL_DEBUG;

    static public void setLevel(byte level) {
        ALog.level = level;
    }

    static public byte getLevel(){
        return level;
    }

    static private IALogCloud sALogCloud;
    static public void setALogCloud(IALogCloud aLogCloud){
        sALogCloud = aLogCloud;
        sALogCloud.setLevel(level);
    }
    /**
     * 上云设置
     * @param endPoint
     * @param accessKeyID
     * @param accessKeySecret
     * @param projectName
     */
    static public void configCloudLog(String endPoint, String accessKeyID, 
    		String accessKeySecret, String projectName){
        if (null != sALogCloud){
            sALogCloud.configCloudLog(endPoint, accessKeyID, accessKeySecret, projectName);
        }
    }

    /**
     * 用户指定该条log是否上云
     * @param tag
     * @param msg
     * @param upToCloud
     */
    static public void d(String tag, String msg, boolean upToCloud) {
        if (null != sALogCloud){
            sALogCloud.d(tag, msg, upToCloud);
        }else {
            log(LEVEL_DEBUG, tag, msg, upToCloud);
        }
    }

    static public void i(String tag, String msg, boolean upToCloud) {
        if (null != sALogCloud){
            sALogCloud.i(tag, msg, upToCloud);
        }else {
            log(LEVEL_INFO, tag, msg, upToCloud);
        }
    }

    static public void w(String tag, String msg, boolean upToCloud) {
        if (null != sALogCloud){
            sALogCloud.w(tag, msg, upToCloud);
        }else {
            log(LEVEL_WARNING, tag, msg, upToCloud);
        }
    }

    static public void e(String tag, String msg, boolean upToCloud) {
        if (null != sALogCloud){
            sALogCloud.e(tag, msg, upToCloud);
        }else {
            log(LEVEL_ERROR, tag, msg, upToCloud);
        }
    }

    static public void e(String tag, String where, String msg, boolean upToCloud) {
        if (null != sALogCloud){
            sALogCloud.e(tag, where, msg, upToCloud);
        }else {
            log(LEVEL_ERROR, tag, (null != where ? where : "") + " ERROR: " + msg, upToCloud);
        }
    }

    static public void e(String tag, String where, Exception ex, boolean upToCloud) {
        if (null != sALogCloud){
            sALogCloud.e(tag, where, ex, upToCloud);
        }else {
            if (null != ex) {
                log(LEVEL_ERROR, tag, (null != where ? where : "") + " EXCEPTION: " + ex.getMessage(), upToCloud);
                ex.printStackTrace();
            } else {
                log(LEVEL_ERROR, tag, (null != where ? where : "") + " EXCEPTION: unknown", upToCloud);
            }
        }
    }

    /**
     * 用户不指定该条log是否上云，由规则引擎自主判断。
     * @param tag
     * @param msg
     */
    static public void d(String tag, String msg) {
        if (null != sALogCloud){
            sALogCloud.d(tag, msg);
        }else{
            log(LEVEL_DEBUG, tag, msg);
        }
    }

    static public void i(String tag, String msg) {
        if (null != sALogCloud){
            sALogCloud.i(tag, msg);
        }else {
            log(LEVEL_INFO, tag, msg);
        }
    }

    static public void w(String tag, String msg) {
        if (null != sALogCloud){
            sALogCloud.w(tag, msg);
        }else {
            log(LEVEL_WARNING, tag, msg);
        }
    }

    static public void e(String tag, String msg) {
        if (null != sALogCloud){
            sALogCloud.e(tag, msg);
        }else {
            log(LEVEL_ERROR, tag, msg);
        }
    }

    static public void e(String tag, String where, String msg) {
        if (null != sALogCloud){
            sALogCloud.e(tag, where, msg);
        }else {
            log(LEVEL_ERROR, tag, (null != where ? where : "") + " ERROR: " + msg);
        }
    }

    static public void e(String tag, String where, Exception ex) {
        if (null != sALogCloud){
            sALogCloud.e(tag, where, ex);
        }else {
            if (null != ex) {
                log(LEVEL_ERROR, tag, (null != where ? where : "") + " EXCEPTION: " + ex.getMessage());
                ex.printStackTrace();
            } else {
                log(LEVEL_ERROR, tag, (null != where ? where : "") + " EXCEPTION: unknown");
            }
        }
    }

    static private void log(byte level, String tag, String msg, boolean up) {
        if (ALog.level > level)
            return;

        switch (level) {
            case LEVEL_DEBUG:
                Log.d(tag, msg);
                break;

            case LEVEL_INFO:
                Log.i(tag, msg);
                break;

            case LEVEL_WARNING:
                Log.w(tag, msg);
                break;

            case LEVEL_ERROR:
                Log.e(tag, msg);
                break;

            default:
                break;
        }
    }
    
    static private void log(byte level, String tag, String msg) {
        if (ALog.level > level)
            return;

        switch (level) {
            case LEVEL_DEBUG:
                Log.d(tag, msg);
                break;

            case LEVEL_INFO:
                Log.i(tag, msg);
                break;

            case LEVEL_WARNING:
                Log.w(tag, msg);
                break;

            case LEVEL_ERROR:
                Log.e(tag, msg);
                break;

            default:
                break;
        }
    }
}
